@@ -2,7 +2,7 @@
 // vim:tabstop=4:shiftwidth=4:expandtab:
 
 /*
- * Copyright (C) 2004-2008 Wu Yongwei <adah at users dot sourceforge dot net>
+ * Copyright (C) 2004-2016 Wu Yongwei <adah at users dot sourceforge dot net>
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any
@@ -27,29 +27,50 @@
  */
 
 /**
- * @file    debug_new.cpp
+ * @file  debug_new.cpp
  *
  * Implementation of debug versions of new and delete to check leakage.
  *
- * @version 4.14, 2008/10/20
- * @author  Wu Yongwei
- *
+ * @date  2016-10-14
  */
 
-#include <new>
-#include <assert.h>
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#ifdef __unix__
-#include <alloca.h>
+#include <new>                  // std::bad_alloc/nothrow_t
+#include <assert.h>             // assert
+#include <stdio.h>              // fprintf/stderr
+#include <stdlib.h>             // abort
+#include <string.h>             // strcpy/strncpy/sprintf
+#include "_nvwa.h"              // NVWA macros
+
+#if NVWA_UNIX
+#include <alloca.h>             // alloca
 #endif
-#ifdef _WIN32
-#include <malloc.h>
+#if NVWA_WIN32
+#include <malloc.h>             // alloca
 #endif
-#include "fast_mutex.h"
-#include "static_assert.h"
+
+#if NVWA_LINUX || NVWA_APPLE
+#include <execinfo.h>           // backtrace
+#endif
+
+#if NVWA_WINDOWS
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
+#include <windows.h>            // CaptureStackBackTrace
+#endif
+
+#include "c++11.h"              // _NOEXCEPT/_NULLPTR
+#include "fast_mutex.h"         // nvwa::fast_mutex
+#include "static_assert.h"      // STATIC_ASSERT
+
+#undef  _DEBUG_NEW_EMULATE_MALLOC
+#undef  _DEBUG_NEW_REDEFINE_NEW
+/**
+ * Macro to indicate whether redefinition of \c new is wanted.  Here it
+ * is defined to \c 0 to disable the redefinition of \c new.
+ */
+#define _DEBUG_NEW_REDEFINE_NEW 0
+#include "debug_new.h"
 
 #if !_FAST_MUTEX_CHECK_INITIALIZATION && !defined(_NOTHREADS)
 #error "_FAST_MUTEX_CHECK_INITIALIZATION not set: check_leaks may not work"
@@ -68,7 +89,7 @@
 /**
  * @def _DEBUG_NEW_CALLER_ADDRESS
  *
- * The expression to return the caller address.  #print_position will
+ * The expression to return the caller address.  nvwa#print_position will
  * later on use this address to print the position information of memory
  * operation points.
  */
@@ -76,7 +97,7 @@
 #ifdef __GNUC__
 #define _DEBUG_NEW_CALLER_ADDRESS __builtin_return_address(0)
 #else
-#define _DEBUG_NEW_CALLER_ADDRESS NULL
+#define _DEBUG_NEW_CALLER_ADDRESS _NULLPTR
 #endif
 #endif
 
@@ -102,30 +123,53 @@
  *
  * The length of file name stored if greater than zero.  If it is zero,
  * only a const char pointer will be stored.  Currently the default
- * behaviour is to copy the file name, because I found that the exit
- * leakage check cannot access the address of the file name sometimes
- * (in my case, a core dump will occur when trying to access the file
- * name in a shared library after a \c SIGINT).  The current default
- * value makes the size of new_ptr_list_t 64 on 32-bit platforms.
+ * value is non-zero (thus to copy the file name) on non-Windows
+ * platforms, because I once found that the exit leakage check could not
+ * access the address of the file name on Linux (in my case, a core dump
+ * occurred when check_leaks tried to access the file name in a shared
+ * library after a \c SIGINT).  This value makes the size of
+ * new_ptr_list_t \c 64 on non-Windows 32-bit platforms (w/o stack
+ * backtrace).
  */
 #ifndef _DEBUG_NEW_FILENAME_LEN
+#if NVWA_WINDOWS
+#define _DEBUG_NEW_FILENAME_LEN 0
+#else
 #define _DEBUG_NEW_FILENAME_LEN 44
 #endif
+#endif
 
 /**
  * @def _DEBUG_NEW_PROGNAME
  *
  * The program (executable) name to be set at compile time.  It is
- * better to assign the full program path to #new_progname in \e main
+ * better to assign the full program path to nvwa#new_progname in \e main
  * (at run time) than to use this (compile-time) macro, but this macro
  * serves well as a quick hack.  Note also that double quotation marks
  * need to be used around the program name, i.e., one should specify a
- * command-line option like <code>-D_DEBUG_NEW_PROGNAME=\"a.out\"</code>
- * in \e bash, or <code>-D_DEBUG_NEW_PROGNAME=\"a.exe\"</code> in the
+ * command-line option like <code>-D_DEBUG_NEW_PROGNAME=\\"a.out\"</code>
+ * in \e bash, or <code>-D_DEBUG_NEW_PROGNAME=\\"a.exe\"</code> in the
  * Windows command prompt.
  */
 #ifndef _DEBUG_NEW_PROGNAME
-#define _DEBUG_NEW_PROGNAME NULL
+#define _DEBUG_NEW_PROGNAME _NULLPTR
+#endif
+
+/**
+ * @def _DEBUG_NEW_REMEMBER_STACK_TRACE
+ *
+ * Macro to indicate whether stack traces of allocations should be
+ * included in NVWA allocation information and be printed while leaks
+ * are reported.  Useful to pinpoint leaks caused by strdup and other
+ * custom allocation functions.  It is also very helpful in filtering
+ * out false positives caused by internal STL/C runtime operations.  It
+ * is off by default because it is quite memory heavy.  Set it to \c 1
+ * to make all allocations have stack trace attached, or set to \c 2 to
+ * make only allocations that lack calling source code information (file
+ * and line) have the stack trace attached.
+ */
+#ifndef _DEBUG_NEW_REMEMBER_STACK_TRACE
+#define _DEBUG_NEW_REMEMBER_STACK_TRACE 0
 #endif
 
 /**
@@ -176,57 +220,66 @@
 #endif
 
 #ifdef _MSC_VER
-#pragma warning(disable: 4073)  // #pragma init_seg(lib) used
+#pragma warning(disable: 4074)  // #pragma init_seg(compiler) used
 #pragma warning(disable: 4290)  // C++ exception specification ignored
-#pragma init_seg(lib)
+#if _MSC_VER >= 1400            // Visual Studio 2005 or later
+#pragma warning(disable: 4996)  // Use the `unsafe' strncpy
+#endif
+#pragma init_seg(compiler)
 #endif
-
-#undef  _DEBUG_NEW_EMULATE_MALLOC
-#undef  _DEBUG_NEW_REDEFINE_NEW
-/**
- * Macro to indicate whether redefinition of \c new is wanted.  Here it
- * is defined to \c 0 to disable the redefinition of \c new.
- */
-#define _DEBUG_NEW_REDEFINE_NEW 0
-#include "debug_new.h"
 
 /**
  * Gets the aligned value of memory block size.
  */
-#define align(s) \
+#define ALIGN(s) \
         (((s) + _DEBUG_NEW_ALIGNMENT - 1) & ~(_DEBUG_NEW_ALIGNMENT - 1))
 
+NVWA_NAMESPACE_BEGIN
+
+/**
+ * The platform memory alignment.  The current value works well in
+ * platforms I have tested: Windows XP, Windows 7 x64, and Mac OS X
+ * Leopard.  It may be smaller than the real alignment, but must be
+ * bigger than \c sizeof(size_t) for it work.  nvwa#debug_new_recorder
+ * uses it to detect misaligned pointer returned by `<code>new
+ * NonPODType[size]</code>'.
+ */
+const size_t PLATFORM_MEM_ALIGNMENT = sizeof(size_t) * 2;
+
 /**
  * Structure to store the position information where \c new occurs.
  */
 struct new_ptr_list_t
 {
-    new_ptr_list_t*     next;
-    new_ptr_list_t*     prev;
-    size_t              size;
+    new_ptr_list_t* next;       ///< Pointer to the next memory block
+    new_ptr_list_t* prev;       ///< Pointer to the previous memory block
+    size_t          size;       ///< Size of the memory block
     union
     {
 #if _DEBUG_NEW_FILENAME_LEN == 0
-    const char*         file;
+    const char*     file;       ///< Pointer to the file name of the caller
 #else
-    char                file[_DEBUG_NEW_FILENAME_LEN];
+    char            file[_DEBUG_NEW_FILENAME_LEN]; ///< File name of the caller
 #endif
-    void*               addr;
+    void*           addr;       ///< Address of the caller to \e new
     };
-    unsigned            line      :31;
-    unsigned            is_array  :1;
-    unsigned            magic;
+    unsigned        line   :31; ///< Line number of the caller; or \c 0
+    unsigned        is_array:1; ///< Non-zero iff <em>new[]</em> is used
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+    void**          stacktrace; ///< Pointer to stack trace information
+#endif
+    unsigned        magic;      ///< Magic number for error detection
 };
 
 /**
- * Magic number for error detection.
+ * Definition of the constant magic number used for error detection.
  */
-const unsigned MAGIC = 0x4442474E;
+static const unsigned DEBUG_NEW_MAGIC = 0x4442474E;
 
 /**
  * The extra memory allocated by <code>operator new</code>.
  */
-const int ALIGNED_LIST_ITEM_SIZE = align(sizeof(new_ptr_list_t));
+static const int ALIGNED_LIST_ITEM_SIZE = ALIGN(sizeof(new_ptr_list_t));
 
 /**
  * List of all new'd pointers.
@@ -237,14 +290,17 @@ static new_ptr_list_t new_ptr_list = {
     0,
     {
 #if _DEBUG_NEW_FILENAME_LEN == 0
-        NULL
+        _NULLPTR
 #else
         ""
 #endif
     },
     0,
     0,
-    MAGIC
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+    _NULLPTR,
+#endif
+    DEBUG_NEW_MAGIC
 };
 
 /**
@@ -263,8 +319,8 @@ static fast_mutex new_output_lock;
 static size_t total_mem_alloc = 0;
 
 /**
- * Flag to control whether #check_leaks will be automatically called on
- * program exit.
+ * Flag to control whether nvwa#check_leaks will be automatically called
+ * on program exit.
  */
 bool new_autocheck_flag = true;
 
@@ -290,6 +346,19 @@ FILE* new_output_fp = stderr;
  */
 const char* new_progname = _DEBUG_NEW_PROGNAME;
 
+/**
+ * Pointer to the callback used to print the stack backtrace in case of
+ * a memory problem.  A null value causes the default stack trace
+ * printing routine to be used.
+ */
+stacktrace_print_callback_t stacktrace_print_callback = _NULLPTR;
+
+/**
+ * Pointer to the callback used to filter out false positives from leak
+ * reports.  A null value means the lack of filtering.
+ */
+leak_whitelist_callback_t leak_whitelist_callback = _NULLPTR;
+
 #if _DEBUG_NEW_USE_ADDR2LINE
 /**
  * Tries printing the position information from an instruction address.
@@ -302,7 +371,7 @@ const char* new_progname = _DEBUG_NEW_PROGNAME;
  */
 static bool print_position_from_addr(const void* addr)
 {
-    static const void* last_addr = NULL;
+    static const void* last_addr = _NULLPTR;
     static char last_info[256] = "";
     if (addr == last_addr)
     {
@@ -313,16 +382,22 @@ static bool print_position_from_addr(const void* addr)
     }
     if (new_progname)
     {
+#if NVWA_APPLE
+        const char addr2line_cmd[] = "atos -o ";
+#else
         const char addr2line_cmd[] = "addr2line -e ";
-#if   defined(__CYGWIN__) || defined(_WIN32)
+#endif
+
+#if NVWA_WINDOWS
         const int  exeext_len = 4;
 #else
         const int  exeext_len = 0;
 #endif
-#if  !defined(__CYGWIN__) && defined(__unix__)
+
+#if NVWA_UNIX && !NVWA_CYGWIN
         const char ignore_err[] = " 2>/dev/null";
-#elif defined(__CYGWIN__) || \
-        (defined(_WIN32) && defined(WINVER) && WINVER >= 0x0500)
+#elif NVWA_CYGWIN || \
+        (NVWA_WIN32 && defined(WINVER) && WINVER >= 0x0500)
         const char ignore_err[] = " 2>nul";
 #else
         const char ignore_err[] = "";
@@ -336,7 +411,7 @@ static bool print_position_from_addr(const void* addr)
         strcpy(cmd, addr2line_cmd);
         strcpy(cmd + sizeof addr2line_cmd - 1, new_progname);
         size_t len = strlen(cmd);
-#if   defined(__CYGWIN__) || defined(_WIN32)
+#if NVWA_WINDOWS
         if (len <= 4
                 || (strcmp(cmd + len - 4, ".exe") != 0 &&
                     strcmp(cmd + len - 4, ".EXE") != 0))
@@ -404,36 +479,88 @@ static bool print_position_from_addr(const void*)
  */
 static void print_position(const void* ptr, int line)
 {
-    if (line != 0)          // Is file/line information present?
+    if (line != 0)             // Is file/line information present?
     {
         fprintf(new_output_fp, "%s:%d", (const char*)ptr, line);
     }
-    else if (ptr != NULL)   // Is caller address present?
+    else if (ptr != _NULLPTR)  // Is caller address present?
     {
         if (!print_position_from_addr(ptr)) // Fail to get source position?
             fprintf(new_output_fp, "%p", ptr);
     }
-    else                    // No information is present
+    else                       // No information is present
     {
         fprintf(new_output_fp, "<Unknown>");
     }
 }
 
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+/**
+ * Prints the stack backtrace.
+ *
+ * When nvwa#stacktrace_print_callback is not null, it is used for
+ * printing the stacktrace items.  Default implementation of call stack
+ * printing is very spartan&mdash;only stack frame pointers are
+ * printed&mdash;but even that output is still useful.  Just do address
+ * lookup in LLDB etc.
+ *
+ * @param stacktrace  pointer to the stack trace array
+ */
+static void print_stacktrace(void** stacktrace)
+{
+     if (stacktrace_print_callback == _NULLPTR)
+     {
+         fprintf(new_output_fp, "Stack backtrace:\n");
+         for (size_t i = 0; stacktrace[i] != _NULLPTR; ++i)
+             fprintf(new_output_fp, "%p\n", stacktrace[i]);
+     }
+     else
+     {
+         stacktrace_print_callback(new_output_fp, stacktrace);
+     }
+}
+#endif
+
+/**
+ * Checks whether a leak should be ignored.  Its runtime performance
+ * depends on the callback nvwa#leak_whitelist_callback.
+ *
+ * @param ptr  pointer to a new_ptr_list_t struct
+ * @return     \c true if the leak should be whitelisted; \c false
+ *             otherwise
+ */
+static bool is_leak_whitelisted(new_ptr_list_t* ptr)
+{
+    if (leak_whitelist_callback == _NULLPTR)
+        return false;
+
+    char const* file = ptr->line != 0 ? ptr->file : _NULLPTR;
+    int line = ptr->line;
+    void* addr = ptr->line == 0 ? ptr->addr : _NULLPTR;
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+    void** stacktrace = ptr->stacktrace;
+#else
+    void** stacktrace = _NULLPTR;
+#endif
+
+    return leak_whitelist_callback(file, line, addr, stacktrace);
+}
+
 #if _DEBUG_NEW_TAILCHECK
 /**
  * Checks whether the padding bytes at the end of a memory block is
  * tampered with.
  *
- * @param ptr   pointer to a new_ptr_list_t struct
- * @return      \c true if the padding bytes are untouched; \c false
- *              otherwise
+ * @param ptr  pointer to a new_ptr_list_t struct
+ * @return     \c true if the padding bytes are untouched; \c false
+ *             otherwise
  */
 static bool check_tail(new_ptr_list_t* ptr)
 {
-    const unsigned char* const pointer = (unsigned char*)ptr +
+    const unsigned char* const tail_ptr = (unsigned char*)ptr +
                             ALIGNED_LIST_ITEM_SIZE + ptr->size;
     for (int i = 0; i < _DEBUG_NEW_TAILCHECK; ++i)
-        if (pointer[i] != _DEBUG_NEW_TAILCHECK_CHAR)
+        if (tail_ptr[i] != _DEBUG_NEW_TAILCHECK_CHAR)
             return false;
     return true;
 }
@@ -446,31 +573,35 @@ static bool check_tail(new_ptr_list_t* ptr)
  * @param file      null-terminated string of the file name
  * @param line      line number
  * @param is_array  boolean value whether this is an array operation
- * @return          pointer to the user-requested memory area; \c NULL
- *                  if memory allocation is not successful
+ * @return          pointer to the user-requested memory area; null if
+ *                  memory allocation is not successful
  */
 static void* alloc_mem(size_t size, const char* file, int line, bool is_array)
 {
     assert(line >= 0);
+#if _DEBUG_NEW_TYPE == 1
+    STATIC_ASSERT(_DEBUG_NEW_ALIGNMENT >= PLATFORM_MEM_ALIGNMENT,
+                  Alignment_too_small);
+#endif
     STATIC_ASSERT((_DEBUG_NEW_ALIGNMENT & (_DEBUG_NEW_ALIGNMENT - 1)) == 0,
                   Alignment_must_be_power_of_two);
     STATIC_ASSERT(_DEBUG_NEW_TAILCHECK >= 0, Invalid_tail_check_length);
     size_t s = size + ALIGNED_LIST_ITEM_SIZE + _DEBUG_NEW_TAILCHECK;
     new_ptr_list_t* ptr = (new_ptr_list_t*)malloc(s);
-    if (ptr == NULL)
+    if (ptr == _NULLPTR)
     {
 #if _DEBUG_NEW_STD_OPER_NEW
-        return NULL;
+        return _NULLPTR;
 #else
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
-                "Out of memory when allocating %u bytes\n",
-                size);
+                "Out of memory when allocating %lu bytes\n",
+                (unsigned long)size);
         fflush(new_output_fp);
         _DEBUG_NEW_ERROR_ACTION;
 #endif
     }
-    void* pointer = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
+    void* usr_ptr = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
 #if _DEBUG_NEW_FILENAME_LEN == 0
     ptr->file = file;
 #else
@@ -481,9 +612,38 @@ static void* alloc_mem(size_t size, const char* file, int line, bool is_array)
         ptr->addr = (void*)file;
 #endif
     ptr->line = line;
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+    ptr->stacktrace = _NULLPTR;
+
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE == 2
+    if (line == 0)
+#endif
+    {
+        void* buffer [255];
+        size_t buffer_length = sizeof(buffer) / sizeof(*buffer);
+
+#if NVWA_UNIX
+        int stacktrace_length = backtrace(buffer, int(buffer_length));
+#endif
+
+#if NVWA_WINDOWS
+        USHORT stacktrace_length = CaptureStackBackTrace(
+            0, DWORD(buffer_length), buffer, _NULLPTR);
+#endif
+
+        size_t stacktrace_size = stacktrace_length * sizeof(void*);
+        ptr->stacktrace = (void**)malloc(stacktrace_size + sizeof(void*));
+
+        if (ptr->stacktrace != _NULLPTR)
+        {
+            memcpy(ptr->stacktrace, buffer, stacktrace_size);
+            ptr->stacktrace[stacktrace_length] = _NULLPTR;
+        }
+    }
+#endif
     ptr->is_array = is_array;
     ptr->size = size;
-    ptr->magic = MAGIC;
+    ptr->magic = DEBUG_NEW_MAGIC;
     {
         fast_mutex_autolock lock(new_ptr_lock);
         ptr->prev = new_ptr_list.prev;
@@ -492,16 +652,16 @@ static void* alloc_mem(size_t size, const char* file, int line, bool is_array)
         new_ptr_list.prev = ptr;
     }
 #if _DEBUG_NEW_TAILCHECK
-    memset((char*)pointer + size, _DEBUG_NEW_TAILCHECK_CHAR,
+    memset((char*)usr_ptr + size, _DEBUG_NEW_TAILCHECK_CHAR,
                                   _DEBUG_NEW_TAILCHECK);
 #endif
     if (new_verbose_flag)
     {
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
-                "new%s: allocated %p (size %u, ",
+                "new%s: allocated %p (size %lu, ",
                 is_array ? "[]" : "",
-                pointer, size);
+                usr_ptr, (unsigned long)size);
         if (line != 0)
             print_position(ptr->file, ptr->line);
         else
@@ -509,29 +669,29 @@ static void* alloc_mem(size_t size, const char* file, int line, bool is_array)
         fprintf(new_output_fp, ")\n");
     }
     total_mem_alloc += size;
-    return pointer;
+    return usr_ptr;
 }
 
 /**
  * Frees memory and adjusts pointers.
  *
- * @param pointer   pointer to delete
+ * @param usr_ptr   pointer to the previously allocated memory
  * @param addr      pointer to the caller
  * @param is_array  flag indicating whether it is invoked by a
  *                  <code>delete[]</code> call
  */
-static void free_pointer(void* pointer, void* addr, bool is_array)
+static void free_pointer(void* usr_ptr, void* addr, bool is_array)
 {
-    if (pointer == NULL)
+    if (usr_ptr == _NULLPTR)
         return;
     new_ptr_list_t* ptr =
-            (new_ptr_list_t*)((char*)pointer - ALIGNED_LIST_ITEM_SIZE);
-    if (ptr->magic != MAGIC)
+            (new_ptr_list_t*)((char*)usr_ptr - ALIGNED_LIST_ITEM_SIZE);
+    if (ptr->magic != DEBUG_NEW_MAGIC)
     {
         {
             fast_mutex_autolock lock(new_output_lock);
             fprintf(new_output_fp, "delete%s: invalid pointer %p (",
-                    is_array ? "[]" : "", pointer);
+                    is_array ? "[]" : "", usr_ptr);
             print_position(addr, 0);
             fprintf(new_output_fp, ")\n");
         }
@@ -548,10 +708,10 @@ static void free_pointer(void* pointer, void* addr, bool is_array)
             msg = "delete after new[]";
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
-                "%s: pointer %p (size %u)\n\tat ",
+                "%s: pointer %p (size %lu)\n\tat ",
                 msg,
                 (char*)ptr + ALIGNED_LIST_ITEM_SIZE,
-                ptr->size);
+                (unsigned long)ptr->size);
         print_position(addr, 0);
         fprintf(new_output_fp, "\n\toriginally allocated at ");
         if (ptr->line != 0)
@@ -581,11 +741,14 @@ static void free_pointer(void* pointer, void* addr, bool is_array)
     {
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
-                "delete%s: freed %p (size %u, %u bytes still allocated)\n",
+                "delete%s: freed %p (size %lu, %lu bytes still allocated)\n",
                 is_array ? "[]" : "",
                 (char*)ptr + ALIGNED_LIST_ITEM_SIZE,
-                ptr->size, total_mem_alloc);
+                (unsigned long)ptr->size, (unsigned long)total_mem_alloc);
     }
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+    free(ptr->stacktrace);
+#endif
     free(ptr);
     return;
 }
@@ -598,40 +761,69 @@ static void free_pointer(void* pointer, void* addr, bool is_array)
 int check_leaks()
 {
     int leak_cnt = 0;
+    int whitelisted_leak_cnt = 0;
     fast_mutex_autolock lock_ptr(new_ptr_lock);
     fast_mutex_autolock lock_output(new_output_lock);
     new_ptr_list_t* ptr = new_ptr_list.next;
+
     while (ptr != &new_ptr_list)
     {
-        const char* const pointer = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
-        if (ptr->magic != MAGIC)
+        const char* const usr_ptr = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
+        if (ptr->magic != DEBUG_NEW_MAGIC)
         {
             fprintf(new_output_fp,
                     "warning: heap data corrupt near %p\n",
-                    pointer);
+                    usr_ptr);
         }
 #if _DEBUG_NEW_TAILCHECK
         if (!check_tail(ptr))
         {
             fprintf(new_output_fp,
                     "warning: overwritten past end of object at %p\n",
-                    pointer);
+                    usr_ptr);
         }
 #endif
-        fprintf(new_output_fp,
-                "Leaked object at %p (size %u, ",
-                pointer,
-                ptr->size);
-        if (ptr->line != 0)
-            print_position(ptr->file, ptr->line);
+
+        if (is_leak_whitelisted(ptr))
+        {
+            ++whitelisted_leak_cnt;
+        }
         else
-            print_position(ptr->addr, ptr->line);
-        fprintf(new_output_fp, ")\n");
+        {
+            fprintf(new_output_fp,
+                    "Leaked object at %p (size %lu, ",
+                    usr_ptr,
+                    (unsigned long)ptr->size);
+
+            if (ptr->line != 0)
+                print_position(ptr->file, ptr->line);
+            else
+                print_position(ptr->addr, ptr->line);
+
+            fprintf(new_output_fp, ")\n");
+
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+            if (ptr->stacktrace != _NULLPTR)
+                print_stacktrace(ptr->stacktrace);
+#endif
+        }
+
         ptr = ptr->next;
         ++leak_cnt;
     }
     if (new_verbose_flag || leak_cnt)
-        fprintf(new_output_fp, "*** %d leaks found\n", leak_cnt);
+    {
+        if (whitelisted_leak_cnt > 0)
+        {
+            fprintf(new_output_fp, "*** %d leaks found (%d whitelisted)\n",
+                leak_cnt, whitelisted_leak_cnt);
+        }
+        else
+        {
+            fprintf(new_output_fp, "*** %d leaks found\n", leak_cnt);
+        }
+    }
+
     return leak_cnt;
 }
 
@@ -651,29 +843,29 @@ int check_mem_corruption()
             ptr != &new_ptr_list;
             ptr = ptr->next)
     {
-        const char* const pointer = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
-        if (ptr->magic == MAGIC
+        const char* const usr_ptr = (char*)ptr + ALIGNED_LIST_ITEM_SIZE;
+        if (ptr->magic == DEBUG_NEW_MAGIC
 #if _DEBUG_NEW_TAILCHECK
                 && check_tail(ptr)
 #endif
                 )
             continue;
 #if _DEBUG_NEW_TAILCHECK
-        if (ptr->magic != MAGIC)
+        if (ptr->magic != DEBUG_NEW_MAGIC)
         {
 #endif
             fprintf(new_output_fp,
-                    "Heap data corrupt near %p (size %u, ",
-                    pointer,
-                    ptr->size);
+                    "Heap data corrupt near %p (size %lu, ",
+                    usr_ptr,
+                    (unsigned long)ptr->size);
 #if _DEBUG_NEW_TAILCHECK
         }
         else
         {
             fprintf(new_output_fp,
-                    "Overwritten past end of object at %p (size %u, ",
-                    pointer,
-                    ptr->size);
+                    "Overwritten past end of object at %p (size %lu, ",
+                    usr_ptr,
+                    (unsigned long)ptr->size);
         }
 #endif
         if (ptr->line != 0)
@@ -681,6 +873,12 @@ int check_mem_corruption()
         else
             print_position(ptr->addr, ptr->line);
         fprintf(new_output_fp, ")\n");
+
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE
+        if (ptr->stacktrace != _NULLPTR)
+            print_stacktrace(ptr->stacktrace);
+#endif
+
         ++corrupt_cnt;
     }
     fprintf(new_output_fp, "*** Checking for memory corruption: %d FOUND\n",
@@ -688,13 +886,37 @@ int check_mem_corruption()
     return corrupt_cnt;
 }
 
-void __debug_new_recorder::_M_process(void* pointer)
+/**
+ * Processes the allocated memory and inserts file/line informatin.
+ * It will only be done when it can ensure the memory is allocated by
+ * one of our operator new variants.
+ *
+ * @param usr_ptr  pointer returned by a new-expression
+ */
+void debug_new_recorder::_M_process(void* usr_ptr)
 {
-    if (pointer == NULL)
+    if (usr_ptr == _NULLPTR)
         return;
+
+    // In an expression `new NonPODType[size]', the pointer returned is
+    // not the pointer returned by operator new[], but offset by size_t
+    // to leave room for the size.  It needs to be compensated here.
+    size_t offset = (char*)usr_ptr - (char*)_NULLPTR;
+    if (offset % PLATFORM_MEM_ALIGNMENT != 0) {
+        offset -= sizeof(size_t);
+        if (offset % PLATFORM_MEM_ALIGNMENT != 0) {
+            fast_mutex_autolock lock(new_output_lock);
+            fprintf(new_output_fp,
+                    "warning: memory unaligned; skipping processing (%s:%d)\n",
+                    _M_file, _M_line);
+            return;
+        }
+        usr_ptr = (char*)usr_ptr - sizeof(size_t);
+    }
+
     new_ptr_list_t* ptr =
-            (new_ptr_list_t*)((char*)pointer - ALIGNED_LIST_ITEM_SIZE);
-    if (ptr->magic != MAGIC || ptr->line != 0)
+            (new_ptr_list_t*)((char*)usr_ptr - ALIGNED_LIST_ITEM_SIZE);
+    if (ptr->magic != DEBUG_NEW_MAGIC || ptr->line != 0)
     {
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
@@ -702,6 +924,12 @@ void __debug_new_recorder::_M_process(void* pointer)
                 _M_file, _M_line);
         return;
     }
+    if (new_verbose_flag) {
+        fast_mutex_autolock lock(new_output_lock);
+        fprintf(new_output_fp,
+                "info: pointer %p allocated from %s:%d\n",
+                usr_ptr, _M_file, _M_line);
+    }
 #if _DEBUG_NEW_FILENAME_LEN == 0
     ptr->file = _M_file;
 #else
@@ -709,8 +937,61 @@ void __debug_new_recorder::_M_process(void* pointer)
             [_DEBUG_NEW_FILENAME_LEN - 1] = '\0';
 #endif
     ptr->line = _M_line;
+#if _DEBUG_NEW_REMEMBER_STACK_TRACE == 2
+    free(ptr->stacktrace);
+    ptr->stacktrace = _NULLPTR;
+#endif
 }
 
+/**
+ * Count of source files that use debug_new.
+ */
+int debug_new_counter::_S_count = 0;
+
+/**
+ * Constructor to increment the count.
+ */
+debug_new_counter::debug_new_counter()
+{
+    ++_S_count;
+}
+
+/**
+ * Destructor to decrement the count.  When the count is zero,
+ * nvwa#check_leaks will be called.
+ */
+debug_new_counter::~debug_new_counter()
+{
+    if (--_S_count == 0 && new_autocheck_flag)
+        if (check_leaks())
+        {
+            new_verbose_flag = true;
+#if defined(__GNUC__) && __GNUC__ == 3
+            if (!getenv("GLIBCPP_FORCE_NEW") && !getenv("GLIBCXX_FORCE_NEW"))
+                fprintf(new_output_fp,
+"*** WARNING:  GCC 3 is detected, please make sure the environment\n"
+"    variable GLIBCPP_FORCE_NEW (GCC 3.2 and 3.3) or GLIBCXX_FORCE_NEW\n"
+"    (GCC 3.4) is defined.  Check the README file for details.\n");
+#endif
+        }
+}
+
+NVWA_NAMESPACE_END
+
+#if NVWA_USE_NAMESPACE
+using namespace nvwa;
+#endif // NVWA_USE_NAMESPACE
+
+/**
+ * Allocates memory with file/line information.
+ *
+ * @param size  size of the required memory block
+ * @param file  null-terminated string of the file name
+ * @param line  line number
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient (#_DEBUG_NEW_STD_OPER_NEW is 0)
+ * @throw bad_alloc memory is insufficient (#_DEBUG_NEW_STD_OPER_NEW is 1)
+ */
 void* operator new(size_t size, const char* file, int line)
 {
     void* ptr = alloc_mem(size, file, line, false);
@@ -724,6 +1005,16 @@ void* operator new(size_t size, const char* file, int line)
 #endif
 }
 
+/**
+ * Allocates array memory with file/line information.
+ *
+ * @param size  size of the required memory block
+ * @param file  null-terminated string of the file name
+ * @param line  line number
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient (#_DEBUG_NEW_STD_OPER_NEW is 0)
+ * @throw bad_alloc memory is insufficient (#_DEBUG_NEW_STD_OPER_NEW is 1)
+ */
 void* operator new[](size_t size, const char* file, int line)
 {
     void* ptr = alloc_mem(size, file, line, true);
@@ -737,105 +1028,159 @@ void* operator new[](size_t size, const char* file, int line)
 #endif
 }
 
+/**
+ * Allocates memory without file/line information.
+ *
+ * @param size  size of the required memory block
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient (#_DEBUG_NEW_STD_OPER_NEW is 0)
+ * @throw bad_alloc memory is insufficient (#_DEBUG_NEW_STD_OPER_NEW is 1)
+ */
 void* operator new(size_t size) throw(std::bad_alloc)
 {
     return operator new(size, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
 }
 
+/**
+ * Allocates array memory without file/line information.
+ *
+ * @param size  size of the required memory block
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient (#_DEBUG_NEW_STD_OPER_NEW is 0)
+ * @throw bad_alloc memory is insufficient (#_DEBUG_NEW_STD_OPER_NEW is 1)
+ */
 void* operator new[](size_t size) throw(std::bad_alloc)
 {
     return operator new[](size, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
 }
 
-#if !defined(__BORLANDC__) || __BORLANDC__ > 0x551
-void* operator new(size_t size, const std::nothrow_t&) throw()
+/**
+ * Allocates memory with no-throw guarantee.
+ *
+ * @param size  size of the required memory block
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient
+ */
+void* operator new(size_t size, const std::nothrow_t&) _NOEXCEPT
 {
     return alloc_mem(size, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0, false);
 }
 
-void* operator new[](size_t size, const std::nothrow_t&) throw()
+/**
+ * Allocates array memory with no-throw guarantee.
+ *
+ * @param size  size of the required memory block
+ * @return      pointer to the memory allocated; or null if memory is
+ *              insufficient
+ */
+void* operator new[](size_t size, const std::nothrow_t&) _NOEXCEPT
 {
     return alloc_mem(size, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0, true);
 }
+
+/**
+ * Deallocates memory.
+ *
+ * @param ptr  pointer to the previously allocated memory
+ */
+void operator delete(void* ptr) _NOEXCEPT
+{
+    free_pointer(ptr, _DEBUG_NEW_CALLER_ADDRESS, false);
+}
+
+/**
+ * Deallocates array memory.
+ *
+ * @param ptr  pointer to the previously allocated memory
+ */
+void operator delete[](void* ptr) _NOEXCEPT
+{
+    free_pointer(ptr, _DEBUG_NEW_CALLER_ADDRESS, true);
+}
+
+#if __cplusplus >= 201402L
+// GCC under C++14 wants these definitions
+
+void operator delete(void* ptr, size_t) _NOEXCEPT
+{
+    free_pointer(ptr, _DEBUG_NEW_CALLER_ADDRESS, false);
+}
+
+void operator delete[](void* ptr, size_t) _NOEXCEPT
+{
+    free_pointer(ptr, _DEBUG_NEW_CALLER_ADDRESS, true);
+}
 #endif
 
-void operator delete(void* pointer) throw()
-{
-    free_pointer(pointer, _DEBUG_NEW_CALLER_ADDRESS, false);
-}
-
-void operator delete[](void* pointer) throw()
-{
-    free_pointer(pointer, _DEBUG_NEW_CALLER_ADDRESS, true);
-}
-
-#if HAVE_PLACEMENT_DELETE
-void operator delete(void* pointer, const char* file, int line) throw()
+/**
+ * Placement deallocation function.  For details, please check Section
+ * 5.3.4 of the C++ 1998 or 2011 Standard.
+ *
+ * @param ptr   pointer to the previously allocated memory
+ * @param file  null-terminated string of the file name
+ * @param line  line number
+ *
+ * @see   http://www.csci.csusb.edu/dick/c++std/cd2/expr.html#expr.new
+ * @see   http://wyw.dcweb.cn/leakage.htm
+ */
+void operator delete(void* ptr, const char* file, int line) _NOEXCEPT
 {
     if (new_verbose_flag)
     {
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
                 "info: exception thrown on initializing object at %p (",
-                pointer);
+                ptr);
         print_position(file, line);
         fprintf(new_output_fp, ")\n");
     }
-    operator delete(pointer);
+    operator delete(ptr);
 }
 
-void operator delete[](void* pointer, const char* file, int line) throw()
+/**
+ * Placement deallocation function.  For details, please check Section
+ * 5.3.4 of the C++ 1998 or 2011 Standard.
+ *
+ * @param ptr   pointer to the previously allocated memory
+ * @param file  null-terminated string of the file name
+ * @param line  line number
+ */
+void operator delete[](void* ptr, const char* file, int line) _NOEXCEPT
 {
     if (new_verbose_flag)
     {
         fast_mutex_autolock lock(new_output_lock);
         fprintf(new_output_fp,
                 "info: exception thrown on initializing objects at %p (",
-                pointer);
+                ptr);
         print_position(file, line);
         fprintf(new_output_fp, ")\n");
     }
-    operator delete[](pointer);
-}
-
-void operator delete(void* pointer, const std::nothrow_t&) throw()
-{
-    operator delete(pointer, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
-}
-
-void operator delete[](void* pointer, const std::nothrow_t&) throw()
-{
-    operator delete[](pointer, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
-}
-#endif // HAVE_PLACEMENT_DELETE
-
-int __debug_new_counter::_S_count = 0;
-
-/**
- * Constructor to increment the count.
- */
-__debug_new_counter::__debug_new_counter()
-{
-    ++_S_count;
+    operator delete[](ptr);
 }
 
 /**
- * Destructor to decrement the count.  When the count is zero,
- * #check_leaks will be called.
+ * Placement deallocation function.  For details, please check Section
+ * 5.3.4 of the C++ 1998 or 2011 Standard.
+ *
+ * @param ptr  pointer to the previously allocated memory
  */
-__debug_new_counter::~__debug_new_counter()
+void operator delete(void* ptr, const std::nothrow_t&) _NOEXCEPT
 {
-    if (--_S_count == 0 && new_autocheck_flag)
-        if (check_leaks())
-        {
-            new_verbose_flag = true;
-#if defined(__GNUC__) && __GNUC__ >= 3
-            if (!getenv("GLIBCPP_FORCE_NEW") && !getenv("GLIBCXX_FORCE_NEW"))
-                fprintf(new_output_fp,
-"*** WARNING:  GCC 3 or later is detected, please make sure the\n"
-"    environment variable GLIBCPP_FORCE_NEW (GCC 3.2 and 3.3) or\n"
-"    GLIBCXX_FORCE_NEW (GCC 3.4 and later) is defined.  Check the\n"
-"    README file for details.\n");
-#endif
-        }
+    operator delete(ptr, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
 }
+
+/**
+ * Placement deallocation function.  For details, please check Section
+ * 5.3.4 of the C++ 1998 or 2011 Standard.
+ *
+ * @param ptr  pointer to the previously allocated memory
+ */
+void operator delete[](void* ptr, const std::nothrow_t&) _NOEXCEPT
+{
+    operator delete[](ptr, (char*)_DEBUG_NEW_CALLER_ADDRESS, 0);
+}
+
+// This is to make Doxygen happy
+#undef _DEBUG_NEW_REMEMBER_STACK_TRACE
+#define _DEBUG_NEW_REMEMBER_STACK_TRACE 0
