@@ -20,12 +20,15 @@
 ##
 ## Downsample is by df times (integer) with a filter
 ## flen taps long, fcut normalized cutoff frequency
-## and att dB of stopband attenuation
+## and att dB of stopband attenuation. If att is negative
+## a Blackman window is used.
 
 function ds = idownsample(is, df, flen, fcut, att)
-
 	df = floor(df);
-	ds = fftconv(is, lowpassfir(flen,fcut/df)' .* chebwin(flen,att));
+	if (att < 0)
+		ds = fftconv(is, lowpassfir(flen,fcut/df)' .* blackman(flen));
+	else
+		ds = fftconv(is, lowpassfir(flen,fcut/df)' .* chebwin(flen,att));
+	endif
 	ds = ds(floor(df/2):df:length(ds));
-
 endfunction
\ No newline at end of file
