@@ -27,27 +27,28 @@
 ## fl: filter length used for ETC filtering
 ## at: filter stopband attenuation, negative use blackman
 ## uf: integer upsampling factor for internal computation
-## ft: filter type, 0 = lowpass, 1 = bandpass, 2 = morlet
-## wm: wavelet frequency multiplier for Morlet wavelet, usually 2
+## ft: filter type, 0 = lowpass, 1 = bandpass, 2 = morlet, 3 = constrained gaussian
+## wm: wavelet frequency multiplier for Morlet wavelet, usually 2,
+##     alpha parameter for constrained gausssian, default 2.5
 
 function tf = tfir(is, ic, cp, fp, fl, at, uf, ft, wm);
 	# Extracts the minimum frequency point
 	mfp = min(fp);
-	
+
 	# Computes the upsampling range
 	ib = ic + floor(2 * min(cp) / mfp);
 	ie = ic + ceil(2 * max(cp) / mfp);
-	
+
 	# Rescales the input signal
 	is = is(ib:ie);
 	ic = ic - ib;
-	
+
 	# Makes the upsampling factor an integer
 	uf = round(uf);
-	
-	# Rescales the filter length	
+
+	# Rescales the filter length
 	fl = fl * uf;
-	
+
 	# Perform the initial upsampling
 	if (uf > 1)
 		ul = length(is) * uf;
@@ -57,114 +58,126 @@ function tf = tfir(is, ic, cp, fp, fl, at, uf, ft, wm);
 		ul = length(is);
 		us = is(:);
 	endif
-	
+
 	# Allocates the cumulative IR output matrix
 	nf = length(fp);
 	nc = length(cp);
 	tf = zeros(nf,nc);
-		
+
 	# Check if morlet
 	if (ft == 2)
 		# Compute the sampling frequency needed to get the
 		# desired base wavelet frequency
 		wf = (wm * uf) / mfp;
-				
+
 		# Computes the wavelet length needed to get the desired
 		# base filter length
 		wl  = (fl - 1) / (2 * wf);
 	endif
-	
+
 	# Add the last frequency point if missing
 	if (fp(nf) < 1.0)
 		fp = [fp 1.0];
 	else
 		# Decrease the number of points
 		nf = nf - 1;
-	endif		
+	endif
 
 	# Cumulative IR computation loop
-	for I = 1:nf 
+	for I = 1:nf
 		# Check the filter type
 		if (ft == 0)
 			# Computes the filter center frequency
 			ff = sqrt(fp(I) * fp(I + 1));
-			
+
 			# Computes the effective filter len
 			efl = round(fl * mfp / ff);
-			
+
 			# Lowpass
 			if (at < 0)
 				ifir = lowpassfir(efl,ff/uf)' .* blackman(efl);
 			else
 				ifir = lowpassfir(efl,ff/uf)' .* chebwin(efl,at);
 			endif
-			
+
 			# Computes the compensation factor
 			cf = ff;
 		elseif (ft == 1)
 			# Computes the filter center frequency
 			ff = sqrt(fp(I) * fp(I + 1));
-			
+
 			# Computes the effective filter len
 			efl = round(fl * mfp / ff);
-			
+
 			# Bandpass
 			if (at < 0)
 				ifir = bandpassfir(efl,fp(I)/uf,fp(I + 1)/uf)' .* blackman(efl);
 			else
 				ifir = bandpassfir(efl,fp(I)/uf,fp(I + 1)/uf)' .* chebwin(efl,at);
 			endif
-			
+
 			# Computes the compensation factor
 			cf = fp(I + 1) - fp(I);
 		elseif (ft == 2)
 			# Computes the filter center frequency
 			ff = sqrt(fp(I) * fp(I + 1));
-			
+
 			# Morlet
-			ifir = morletw(wf,mfp / ff,0,wl,1,wm * pi);			
-			
+			ifir = morletw(wf,mfp / ff,0,wl,1,wm * pi);
+
 			# Estract the filter len
 			efl = length(ifir);
-			
+
 			# Computes the compensation factor
 			cf = ff ^ 0.5;
+		elseif (ft == 3)
+			# Computes the filter center frequency
+			ff = sqrt(fp(I) * fp(I + 1));
+
+			# Computes the effective filter len
+			efl = round(fl * mfp / ff);
+
+			# Constrained gaussian sine
+			ifir = sin(2 * pi * ff * (0:(efl - 1)) / efl)' .* acgw(efl,wm);
+
+			# Computes the compensation factor
+			cf = ff;
 		else
 			# Computes the filter center frequency
 			ff = sqrt(fp(I) * fp(I + 1));
-			
+
 			# Computes the effective filter len
 			efl = round(fl * mfp / ff);
-			
+
 			# Lowpass
 			if (at < 0)
 				ifir = lowpassfir(efl,ff/uf)' .* blackman(efl);
-			else			
+			else
 				ifir = lowpassfir(efl,ff/uf)' .* chebwin(efl,at);
 			endif
-			
+
 			# Computes the compensation factor
 			cf = ff;
 		endif
-		
+
 		# Applies the filter
 		if (efl > 1)
 			ec = real(fftconv(ifir,us));
 		else
 			ec = us;
 		endif
-		
+
 		# Computes the filter delay
 		fd = (efl) / 2;
-	
+
 		# Computes the references
 		el = ul + efl - 1;
-		uc = (ic * uf) + fd;		
-		
+		uc = (ic * uf) + fd;
+
 		# Computes the cicle rescaling
 		cs = (-uc * ff) / (2 * uf);
 		ce = ((el - uc) * ff) / (2 * uf);
-				
+
 		# Interpolate at the requested cycle points
 		tf(I,:) = interp1(linspace(cs,ce,el), ec, cp, '*pchip') / cf;
 	endfor
