@@ -1,7 +1,7 @@
 /****************************************************************************
 
     DRC: Digital Room Correction
-    Copyright (C) 2002-2010 Denis Sbragion
+    Copyright (C) 2002-2017 Denis Sbragion
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 		You can contact the author on Internet at the following address:
 
-				d.sbragion@infotecna.it
+				d.sbragion@neomerica.it
 
 		This program uses the FFT  routines from  Takuya Ooura and the GNU
 		Scientific  Library (GSL).  Many thanks  to Takuya Ooura and the GSL
@@ -54,8 +54,8 @@
 #include <fenv.h>
 
 /* Versione corrente */
-#define DRCVersion "3.2.1"
-#define DRCCopyright "2002-2012"
+#define DRCVersion "3.2.2"
+#define DRCCopyright "2002-2017"
 
 /* Memory leaks debugger */
 #ifdef DebugMLeaks
@@ -97,7 +97,7 @@ void ShowDRCUsage(void)
 		sputs("\nParameters:\n");
 		sputs("  --help : show the full options list (long)");
 		sputs("  Options: parameters overwriting options");
-		sputs("  DRCFile: name of DRC configration file\n");
+		sputs("  DRCFile: name of DRC configuration file\n");
 		sputs("  Refer to the manual and samples for options");
 		sputs("  details and file format\n");
 	}
@@ -106,7 +106,7 @@ void ShowDRCUsage(void)
 int main(int argc, char * argv[])
 	{
 		/* Segnale in ingresso */
-		DLReal * OInSig;
+		DLReal * OInSig = NULL;
 		DLReal * InSig;
 
 		/* Punto iniziale e finale lettura da file */
@@ -122,16 +122,16 @@ int main(int argc, char * argv[])
 		int WLen1;
 		int WStart2;
 		int WLen2;
-		int WStart3;
-		int WLen3;
+		int WStart3 = 0;
+		int WLen3 = 0;
 
 		/* Array correzione microfono */
 		DLReal * MCFilterFreqs;
 		DLReal * MCFilterM;
 		DLReal * MCFilterP;
-		DLReal * MCFilter;
+		DLReal * MCFilter = NULL;
 		DLReal * MCOutSig;
-		int MCOutSigStart;
+		int MCOutSigStart = 0;
 		int MCOutSigLen;
 		int MCMPFLen;
 
@@ -142,7 +142,7 @@ int main(int argc, char * argv[])
 		int EPPFSigLen;
 
 		/* Array convoluzione MP/EP */
-		DLReal * MPEPSig;
+		DLReal * MPEPSig = NULL;
 		int MPEPSigLen;
 
 		/* Array inversione impulso */
@@ -151,12 +151,12 @@ int main(int argc, char * argv[])
 		int PTTConvStart;
 		int PTTRefLen;
 		DLReal * PTFilter;
-		MKSETFType TFType;
+		MKSETFType TFType = MKSETFLinearPhase;
 
 		/* Array calcolo risposta target psicoacustica*/
 		DLReal * ISRevSig;
 		DLReal * ISMPEPSig;
-		DLReal * ISRevOut;
+		DLReal * ISRevOut = NULL;
 		int ISSigLen;
 
 		/* Array troncatura ringing */
@@ -167,7 +167,7 @@ int main(int argc, char * argv[])
 		DLReal * PSFilterFreqs;
 		DLReal * PSFilterM;
 		DLReal * PSFilterP;
-		DLReal * PSFilter;
+		DLReal * PSFilter = NULL;
 		DLReal * PSOutSig;
 		int PSOutSigLen;
 		int PSMPFLen;
@@ -184,7 +184,7 @@ int main(int argc, char * argv[])
 		int J;
 
 		/* Tipo interpolazione filtri target */
-		InterpolationType FIType;
+		InterpolationType FIType = Linear;
 
 		/* Tipo funzione di prefiltratura */
 		SLPPrefilteringType SLPType;
@@ -347,7 +347,7 @@ int main(int argc, char * argv[])
 
 				/* Verifica che la finestratura sia corretta */
 				if ((Cfg.BCInitWindow / 2 - Cfg.BCPreWindowLen) < PSStart)
-					sputs("!!Warning: input signal too short for correct signal prewindowing, spurios spikes may be generated.");
+					sputs("!!Warning: input signal too short for correct signal prewindowing, spurious spikes may be generated.");
 
 				for (I = 0;I < Cfg.BCInitWindow / 2 - Cfg.BCPreWindowLen;I++)
 					InSig[I] = 0;
@@ -672,7 +672,7 @@ int main(int argc, char * argv[])
 			}
 
     /* Calcola il valore RMS del segnale in ingresso */
-		SRMSValue = GetRMSLevel(OInSig,MCOutSigLen);
+		SRMSValue = GetRMSLevel(&(MCOutSig[MCOutSigStart]),MCOutSigLen);
 		if (SRMSValue > ((DLReal) 0.0))
 			printf("Input signal RMS level %f (%f dB).\n",(double) SRMSValue, (double) (20 * log10((double) SRMSValue)));
 		else
@@ -854,7 +854,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if (((Cfg.BCPreWindowLen == 0) && (WStart1 < PSStart)) || ((WStart1 + Cfg.MPLowerWindow) > PSEnd))
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					BWPreFilt(&MPSig[WStart1],Cfg.MPLowerWindow,Cfg.MPUpperWindow,
 						Cfg.MPFilterLen,Cfg.MPBandSplit,Cfg.MPWindowExponent,
@@ -867,7 +867,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if ((WStart1 + Cfg.MPLowerWindow) > PSEnd)
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					BWPreFilt(&MPSig[WStart1],Cfg.MPLowerWindow,Cfg.MPUpperWindow,
 						Cfg.MPFilterLen,Cfg.MPBandSplit,Cfg.MPWindowExponent,
@@ -880,7 +880,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if (((Cfg.BCPreWindowLen == 0) && (WStart1 < PSStart)) || ((WStart1 + Cfg.MPLowerWindow) > PSEnd))
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					SLPreFilt(&MPSig[WStart1],Cfg.MPLowerWindow,Cfg.MPUpperWindow,
 						Cfg.MPFilterLen,Cfg.MPBandSplit,Cfg.MPWindowExponent,
@@ -893,7 +893,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if ((WStart1 + Cfg.MPLowerWindow) > PSEnd)
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					SLPreFilt(&MPSig[WStart1],Cfg.MPLowerWindow,Cfg.MPUpperWindow,
 						Cfg.MPFilterLen,Cfg.MPBandSplit,Cfg.MPWindowExponent,
@@ -1113,7 +1113,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if (((Cfg.BCPreWindowLen == 0) && (WStart2 < PSStart)) || ((WStart2 + Cfg.EPLowerWindow) > PSEnd))
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					BWPreFilt(&EPSig[WStart2],Cfg.EPLowerWindow,Cfg.EPUpperWindow,
 						Cfg.EPFilterLen,Cfg.EPBandSplit,Cfg.EPWindowExponent,
@@ -1126,7 +1126,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if ((WStart2 + Cfg.EPLowerWindow) > PSEnd)
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					BWPreFilt(&EPSig[WStart2],Cfg.EPLowerWindow,Cfg.EPUpperWindow,
 						Cfg.EPFilterLen,Cfg.EPBandSplit,Cfg.EPWindowExponent,
@@ -1139,7 +1139,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if (((Cfg.BCPreWindowLen == 0) && (WStart2 < PSStart)) || ((WStart2 + Cfg.EPLowerWindow) > PSEnd))
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					SLPreFilt(&EPSig[WStart2],Cfg.EPLowerWindow,Cfg.EPUpperWindow,
 						Cfg.EPFilterLen,Cfg.EPBandSplit,Cfg.EPWindowExponent,
@@ -1152,7 +1152,7 @@ int main(int argc, char * argv[])
 
 					/* Verifica che la finestratura sia corretta */
 					if ((WStart2 + Cfg.EPLowerWindow) > PSEnd)
-						sputs("!!Warning: input signal too short for correct signal prefiltering, spurios spikes may be generated.");
+						sputs("!!Warning: input signal too short for correct signal prefiltering, spurious spikes may be generated.");
 
 					SLPreFilt(&EPSig[WStart2],Cfg.EPLowerWindow,Cfg.EPUpperWindow,
 						Cfg.EPFilterLen,Cfg.EPBandSplit,Cfg.EPWindowExponent,
