@@ -19,7 +19,7 @@
 
 		You can contact the author on Internet at the following address:
 
-				d.sbragion@infotecna.it
+				d.sbragion@neomerica.it
 
 ****************************************************************************/
 
@@ -33,6 +33,7 @@
 ****************************************************************************/
 
 #include "fft.h"
+#include <stdlib.h>
 
 /* Memory leaks debugger */
 #ifdef DebugMLeaks
