@@ -364,6 +364,10 @@ static const char *
      char *const *argv;
      const char *optstring;
 {
+	/* Get rid of unused argument warnings */
+	(void) argc;
+	(void) argv;
+
 	/* Start processing options with ARGV-element 1 (since ARGV-element 0
 	   is the program name); the sequence of previously skipped
 	   non-option ARGV-elements is empty.  */
