@@ -19,7 +19,7 @@
 
 		You can contact the author on Internet at the following address:
 
-				d.sbragion@infotecna.it
+				d.sbragion@neomerica.it
 
 ****************************************************************************/
 
@@ -315,7 +315,7 @@ Boolean LPDipLimit(DLReal * Sig,const int SigLen,const DLReal MinGain,const DLRe
 		DLReal DLSLevel;
 		DLReal DLLevel;
 		DLReal DLGFactor;
-		DLReal DLMFactor;
+		DLReal DLMFactor = 0;
 		DLReal DLAbs;
 		DLReal DLMin;
 
@@ -524,7 +524,7 @@ Boolean HMPDipLimit(DLReal * Sig,const int SigLen,const DLReal MinGain,const DLR
 		DLReal DLSLevel;
 		DLReal DLLevel;
 		DLReal DLGFactor;
-		DLReal DLMFactor;
+		DLReal DLMFactor = 0;
 		DLReal DLAbs;
 		DLReal DLMin;
 		Boolean LogLimit;
@@ -846,7 +846,7 @@ Boolean LPPeakLimit(DLReal * Sig,const int SigLen,const DLReal MaxGain,const DLR
 		DLReal PLSLevel;
 		DLReal PLLevel;
 		DLReal PLGFactor;
-		DLReal PLMFactor;
+		DLReal PLMFactor = 0;
 		DLReal PLAbs;
 		DLReal PLMax;
 
@@ -1055,7 +1055,7 @@ Boolean HMPPeakLimit(DLReal * Sig,const int SigLen,const DLReal MaxGain,const DL
 		DLReal PLSLevel;
 		DLReal PLLevel;
 		DLReal PLGFactor;
-		DLReal PLMFactor;
+		DLReal PLMFactor = 0;
 		DLReal PLAbs;
 		DLReal PLMax;
 		Boolean LogLimit;
