@@ -91,7 +91,7 @@
 enum quote_option {
   QUOTE_NONE,
   QUOTE_ENQUOTE,
-  QUOTE_DEQUOTE,
+  QUOTE_DEQUOTE
 };
 
 static TCHAR *skipleading(const TCHAR *str)
